﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GameClient : MonoBehaviour
{

    RTNetworkClient _client;

    public enum PacketType
    {
        Text = RTNetworkManager.PacketType.RESERVED+1
    }
    // Use this for initialization

        public RTNetworkClient GetNetworkClient() { return _client; }

    void Awake ()
    {
        _client = GetComponent<RTNetworkClient>();

        _client.OnPacketEvent += OnPacketEvent;
        _client.OnPacketStringEvent += OnPacketStringEvent;
        _client.OnConnectedEvent += OnClientConnected;
        _client.OnConnectFailedEvent += OnClientConnectFailed;
        _client.OnDisconnectedEvent += OnClientDisconnected;
        _client.OnPacketPingEvent += OnPacketPingEvent;
    }

    void Start()
    {
        RTConsole.Get().OnGotConsoleInputEvent += OnGotConsoleInputEvent;

    }

    // Update is called once per frame
    void Update ()
    {
		
	}

    void OnPacketStringEvent(string text)
    {
        RTConsole.Log(text);
    }

    void OnPacketEvent(short packetType, byte[] buffer, int size, int index)
    {
        print("Got " + packetType);
    }

    public void Connect(string serverAddress, int serverUDPPort)
    {
        _client.Connect(serverAddress, serverUDPPort);
    }

    public void Disconnect()
    {
        RTConsole.Log("Manually disconnecting client.");

        _client.Disconnect();
    }
    
    void OnClientConnected()
    {
        RTConsole.Log("Main client connected to server. ("+GameLogic.Get().GetServerAddress()+")");
    }

    public void SetShowDebugMessages(bool bNew)
    {
        _client.SetShowDebugMessages(bNew);
    }

    void OnClientConnectFailed()
    {
        RTConsole.Log("Client failed to connect.  Try refreshing to try again?  Server might be down.");
    }

    void OnClientDisconnected()
    {
        RTConsole.Log("Client disconnected.  Click Connect UDP or Connect WebGL to try to connect again.");
        //GameLogic.Get().DoClientConnect();
    }

    void OnGotConsoleInputEvent(string text)
    {
        //RTConsole.Log("Got " + text);

        if (!_client.isConnected)
        {
            RTConsole.Log("Main client is `4not connected`` to a server.  Connect first.");
            return;
        }
        RTNetworkManager.Get().SendString(_client, text);
    }

    void OnPacketPingEvent(float pingTime)
    {
        RTConsole.Log("Client ping: `$" + pingTime * 1000.0f + "`` ms");
    }

    public void Ping()
    {
        if (!_client.isConnected)
        { 
            RTConsole.Log("Main client is not connected to a server, can't ping. Click Connect UDP or Connect WebGL");
            return;
        }

        // RTConsole.Log("Client ping test");
        if (!_client.IsWaitingForPing())
        _client.Ping();
    }

}
