﻿/*
 * Written by Seth A. Robinson (rtsoft.com)
 * 
*/
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class StressTester : MonoBehaviour
{

    LinkedList<RTNetworkClient> _clients;

    bool _IsDoingChatTest = false;
    float _chatStressTimer;

    int _clientsToAdd = 0;
    int _clientsToRemove = 0;
    int stressBytesReceived = 0;
    int stressLinesOfText = 0;
    float showStatusTimer = 0;
    int stressSentLines = 0;
    int stressSentLinesSinceLastDisplay = 0;
    int stressBytesReceivedSinceLastDisplay = 0;
    int stressLinesOfTextSinceLastDisplay = 0;


    // Use this for initialization
    void Start ()
    {
        _clients = new LinkedList<RTNetworkClient>();
	}
	
    public void Spinup(int clientCount)
    {
        _clientsToAdd += clientCount;
    }

    void AddClient()
    {

        _clientsToAdd--;

        RTNetworkClient client = gameObject.AddComponent<RTNetworkClient>();
        _clients.AddLast(client);

        /*
        //more events to grab if we cared
        client.OnPacketEvent += OnPacketEvent;
        client.OnConnectedEvent += OnClientConnected;
        client.OnDisconnectedEvent += OnClientDisconnected;
        */

        //client.SetShowDebugMessages(true);
        client.OnPacketPingEvent += OnPacketPingEvent;
        client.OnPacketStringEvent += OnPacketStringEvent;

        client.OnConnectFailedEvent += OnClientConnectFailed;

        client.Connect(GameLogic.Get().GetServerAddress(), GameLogic.Get().GetCorrectServerPort());
        print("Stress test client added, there are now "+ _clients.Count);

    }
    public void SpinDown(int clientCount)
    {
        _clientsToRemove += clientCount;

    }

    void RemoveClient()
    {
        _clientsToRemove--;

      if (_clients.Count > 0)
        {
            GameObject.Destroy(_clients.First.Value);
            _clients.RemoveFirst();
        }

        print("Stress test client removed, there are now " + _clients.Count);

    }

    void OnClientConnectFailed()
    {
        RTConsole.Log("Client failed to connect");
    }

    public void Ping()
    {
        foreach (RTNetworkClient client in _clients)
        {
            if (!client.IsWaitingForPing())
                client.Ping();
        }
    }

    void OnPacketPingEvent(float pingTime)
    {
        RTConsole.Log("Stress tester ping: `$" + pingTime * 1000.0f + "`` ms");
    }

    // Update is called once per frame

    void OnPacketStringEvent(string text)
    {
        //RTConsole.LogRaw("'"+text+"' ");
        stressLinesOfText++;
        stressBytesReceived += text.Length;

        stressLinesOfTextSinceLastDisplay++;
        stressBytesReceivedSinceLastDisplay += text.Length;
    }

    public void ToggleChatStressTest()
    {
        
        _IsDoingChatTest = !_IsDoingChatTest;

        if (_IsDoingChatTest)
        {
            RTConsole.Log("`2Enabling sending a random chat string every second to all "+_clients.Count+" stress test connections.``");
        } else
        {
            RTConsole.Log("`2Disabled chat stress test.`` "+_clients.Count+"`` clients still initialized.  We sent "+stressSentLines+" lines of text and received "+stressLinesOfText+" lines ("+stressBytesReceived+" bytes)");

        }

        stressBytesReceived = 0;
        stressLinesOfText = 0;
        stressSentLines = 0;
        stressLinesOfTextSinceLastDisplay = 0;
        stressBytesReceivedSinceLastDisplay = 0;


    }

    string GenerateRandomText(int words)
    {
        string crap = "";
        while (words > 0)
        {
            words--;
            int r = Random.Range(0, 10);

            if (crap.Length != 0) crap = crap + " ";

            switch (r)
            {
                case 0: crap = crap + "small"; break;
                case 1: crap = crap + "big"; break;
                case 2: crap = crap + "you"; break;
                case 3: crap = crap + "are"; break;
                case 4: crap = crap + "a"; break;
                case 5: crap = crap + "crazy"; break;
                case 6: crap = crap + "person"; break;
                case 7: crap = crap + "love"; break;
                case 8: crap = crap + "smallwood"; break;
                case 9: crap = crap + "will"; break;
                case 10: crap = crap + "your"; break;


                default: break;
            }

        }

        return crap;

    }

   void Update()
    {

        if (_IsDoingChatTest && _chatStressTimer < Time.time)
        {

            foreach (RTNetworkClient client in _clients)
            {
                stressSentLines++;
                stressSentLinesSinceLastDisplay++;
                RTNetworkManager.Get().SendString(client, GenerateRandomText(Random.Range(10, 30)));
            }

            _chatStressTimer = Time.time + 1.0f;

        }

        if (_clientsToAdd > 0) AddClient();
        if (_clientsToRemove > 0) RemoveClient();

        if (Time.time - showStatusTimer > 1 && (stressBytesReceivedSinceLastDisplay > 0 || stressSentLinesSinceLastDisplay > 0))
        {
            RTConsole.Log("`wTime: "+Time.time.ToString("0.#")+" Stress Clients Active: "+_clients.Count+" Sent "+stressSentLinesSinceLastDisplay+" lines of text, received " + stressLinesOfTextSinceLastDisplay + " lines of text (" + stressBytesReceivedSinceLastDisplay + " bytes)``");
            showStatusTimer = Time.time;
            stressSentLinesSinceLastDisplay = 0;
            stressLinesOfTextSinceLastDisplay = 0;
            stressBytesReceivedSinceLastDisplay = 0;
            
        }

    }



}

