﻿using System.Collections;
using System.Collections.Generic;
using System;
using System.IO;
using System.Text;
using System.Net;
using System.Net.Sockets;
using UnityEngine;
using System.Threading;

//Let's use TcpListener to grab connecting people and plug them into an RTSocket to talk with via a standard telnet style connection

public class TelnetHost : MonoBehaviour
{
    private TcpListener server = null;
    RTSocket client = null;
    LinkedList<string> _lines = new LinkedList<string>();

    // Use this for initialization
    void Start()
    {

    }

    void OnApplicationQuit()
    {
        // Make sure prefs are saved before quitting.
        //PlayerPrefs.Save();
      if (client != null)
        {
            client.SocketKill();
            client = null;
            print("Closing telnet host");

        }
    }

    public bool Setup(int port)
    {
       
        if (server != null)
        {
            server.Stop();
            server = null;

        }

        server = new TcpListener(IPAddress.Parse("0.0.0.0"), port); //IPAdress.Loopback binds to 127.0.01 on my linux server which doesn't work, this one does
        
        // Start listening for client requests.
        server.Start();

        return true;
    }
    
	
    void ProcessLine(string text)
    {
        string s;

        if (text == "/stats")
        {
            s = RTNetworkManager.Get().GetStats() + "\r\n" + RTNetworkServer.Get().GetStats();
        }
        else if (text == "/?" || text == "?")
        {
            s = "Telnet admin - Allows you to monitor the server and issue commands from any telnet client\r\n\r\n";

            s = "/stats - Show stats, if we had any\r\n";
            s += "/quit - log us off";
        }
        else
        {
            s = "Unhandled command " + text;
        }

        s = s + GetPrompt();
        client.WriteString(s);
    }

	// Update is called once per frame
	void Update ()
    {

        //process any text we've received

        if (_lines.Count > 0)
        {
            ProcessLine(_lines.First.Value);
            _lines.RemoveFirst();
        }

        if (server == null ) return;
        
        try
        {
            if (server.Pending())
            {
                if (client != null)
                {
                    RTConsole.Log("Uh oh, we already have a telnet connection, kicking it.");
                    client.WriteString("Someone else is logging in so we're kicking you, sorry.");
                    client.SocketKill();
                    client = null;
                }

                RTConsole.Log("Incoming telnet connection...");
                //someone wants to connect
                client = new RTSocket();
                client.OnTelnetStringReceivedEvent += OnTelnetStringReceivedEvent;
                client.SetSocket(server.AcceptTcpClient());
                client.WriteString("Welcome to the NetTest server admin telnet.  ? For help");
                client.WriteString(GetPrompt());
            }

        }
        catch (Exception e)
        {
            if (e == null)
            {
                //just doing something to get rid of the declared but not used issue
            }

            RTConsole.Log("Can't read from telnet admin port");
            server = null;
            return;
        }
    }

    string GetPrompt()
    {
        return "\r\n\r\nCommand > ";
    }

    void OnTelnetStringReceivedEvent( string text)
    {

        _lines.AddLast(text); //deal with it later, we're being called from a thread maybe.  (yes on Win64, no on linux?)
        //RTConsole.Log("OnTelnetStringReceivedEvent: Got " + text);

    }


}
